#!/bin/bash

error() {
	echo >&2 "$1"
	echo >&2 "****ERROR****: $2"
	if [ "$3"x == "return"x ]; then
		return $2
	else
		exit $2
	fi
}

testService() {
	# 输出: pingStatus
	pingStatus="false"
	nslookup "$1" >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		pingStatus=false
	else
		pingStatus=true
	fi
}

echo "开始企业服务初始化..."
source $JUSTEP_HOME/conf/env.sh

# 企业应用DOMAIN_PREFIX的格式: ${serviceName}${tenantCode}${tenantCode.len.encode1size}
domain=$DOMAIN_PREFIX
size=${domain: -1}
size=$(printf "%d" \'$size)
if [ "$size" -gt "48" ] && [ "$size" -le "57" ]; then
	size=$(expr $size - 48)
elif [ "$size" -eq "48" ]; then
	size=10
else
	size=$(expr $size - 97 + 11)
fi
index=$(expr ${#domain} - $size - 1)

serviceName=${domain:0:$index}

if [ -z "$POOL_TYPE" ] && [ -n "$NAMESPACE" ]; then
	serviceNamespace=$NAMESPACE
else
	serviceNamespace="$NAMESPACE_PREFIX"tenant-"$TENANT_CODE"
fi
servieNanagerUrl="http://entry.$serviceNamespace"
echo DOMAIN_PREFIX: $DOMAIN_PREFIX serviceName: $serviceName, servieNanagerUrl: $servieNanagerUrl

# 如果entry服务存在，表示是企业应用
testService entry.$serviceNamespace
if [ "$pingStatus"x == "false"x ]; then
	echo "非企业租户应用，忽略企业服务初始化"
	exit
fi

# 组织数据导入
echo "开始组织数据导入..."
if [ -d "$JUSTEP_HOME/model/service/service-meta-info/data" ]; then
	opmFiles=$(find $JUSTEP_HOME/model/service/service-meta-info/data/ -name "opm.*.json")
else
	opmFiles=""
fi
if [ -n "$opmFiles" ]; then
	for opmFile in ${opmFiles[@]}; do
		curl -X POST -H "Content-Type: application/json" "$servieNanagerUrl/authorize/debug/import" -d@$opmFile
		if [ 0 -eq $? ]; then
			echo "导入: $opmFile..."
		else
			error "导入失败: $opmFile" 1
		fi
	done
else
	echo "无组织数据定义，忽略组织数据导入"
fi
echo "组织数据导入结束"

# 流程导入
wfRoute=$(curl -s --url $servieNanagerUrl/manager/services/route | jq -r .wf)
if [ -n "$wfRoute" ] && [ "$wfRoute" != "null" ]; then
	echo "开始流程导入..."
	wfUrl="http://wf.$serviceNamespace"
	processFile=$JUSTEP_HOME/model/service/service-meta-info/data/process.json
	if [ -f "$processFile" ]; then
		process=/SA/wf/manager/managerProcess
		activity=make
		action=publishProcessDefineAction

		json=$(curl -s -X POST "$wfUrl/x5/UI2/SA/wf/service/importProcess.j" -H "Content-Type:multipart/form-data" -H "Accept:application/json" --form "process=$process" --form "activity=$activity" --form "action=$action" --form "define=@$processFile")
		if [ "$(echo $json | jq -r .flag)"x != "true"x ]; then
			error "导入失败: $processFile。错误信息: $json" 1
		fi
	else
		echo "无工作流定义，忽略流程导入"
	fi
	echo "流程导入结束"
else
	echo "无工作流服务，忽略流程导入"
fi

echo "企业服务初始化结束"
