#!/bin/bash

error() {
	echo >&2 "$1"
	echo >&2 "****ERROR****: $2"
	if [ "$3"x == "return"x ]; then
		return $2
	else
		exit $2
	fi
}

testService() {
	# 输出: pingStatus
	pingStatus="false"
	nslookup "$1" >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		pingStatus=false
	else
		pingStatus=true
	fi
}

echo "开始企业服务初始化..."
source $JUSTEP_HOME/conf/env.sh

# 企业应用DOMAIN_PREFIX的格式: ${serviceName}${tenantCode}${tenantCode.len.encode1size}
domain=$DOMAIN_PREFIX
size=${domain: -1}
size=$(printf "%d" \'$size)
if [ "$size" -gt "48" ] && [ "$size" -le "57" ]; then
	size=$(expr $size - 48)
elif [ "$size" -eq "48" ]; then
	size=10
else
	size=$(expr $size - 97 + 11)
fi
index=$(expr ${#domain} - $size - 1)

serviceName=${domain:0:$index}

if [ -z "$POOL_TYPE" ] && [ -n "$NAMESPACE" ]; then
	serviceNamespace=$NAMESPACE
else
	serviceNamespace="$NAMESPACE_PREFIX"tenant-"$TENANT_CODE"
fi
servieNanagerUrl="http://entry.$serviceNamespace"
echo DOMAIN_PREFIX: $DOMAIN_PREFIX serviceName: $serviceName, servieNanagerUrl: $servieNanagerUrl

# 如果entry服务存在，表示是企业应用
testService entry.$serviceNamespace
if [ "$pingStatus"x == "false"x ]; then
	echo "非企业租户应用，忽略企业服务初始化"
	exit
fi

# internal类型的服务自注册
if [ "$SERVICE_TYPE"x == "internal"x ]; then
	echo "开始注册服务$serviceName..."
	for i in {1..3}; do
		httpResponse=$(curl -s --url "$servieNanagerUrl/manager/services" \
			-H 'Content-Type: application/json' \
			--data-raw '{"address":"http://'$serviceName.$serviceNamespace'","label":"'$serviceName'","type":"internal"}')
		# curl post请求不能使用-I获取，这里通过结果来判断正确与否，正确时候不会有status信息
		httpCode=$(echo $httpResponse | jq .status)
		if [ -n "$httpCode" ] && [ "$httpCode" != "null" ]; then
			if [ $i -ge 3 ]; then
				error "注册服务$serviceName失败[$httpResponse]" 1
			else
				echo "注册服务$serviceName失败[$httpResponse]，10秒后重试..."
				sleep 10
			fi

		else
			echo "注册服务$serviceName结束"
			break
		fi
	done
fi

# 企业服务刷新
echo "开始刷新服务$serviceName..."
for i in {1..3}; do
	httpCode=$(curl -I -s --url $servieNanagerUrl/manager/services/refresh/$serviceName -w %{http_code} | tail -n1)
	if [ "$httpCode"x != "200"x ]; then
		if [ $i -ge 3 ]; then
			error "刷新服务$serviceName失败[$httpCode]" 1
		else
			echo "刷新服务$serviceName失败[$httpCode]，10秒后重试..."
			sleep 10
		fi
	else
		echo "刷新服务$serviceName结束"
		break
	fi
done

# 工作流服务刷新，兼容性保留
wfRoute=$(curl -s --url $servieNanagerUrl/manager/services/route | jq -r .wf)
if [ -n "$wfRoute" ] && [ "$wfRoute" != "null" ]; then
	echo "开始刷新工作流服务..."
	httpCode=$(curl -I -s --url $servieNanagerUrl/manager/services/refresh/wf -w %{http_code} | tail -n1)
	if [ "$httpCode"x != "200"x ]; then
		error "刷新工作流服务失败[$httpCode]" 2
	else
		echo "刷新工作流服务结束"
	fi
else
	echo "无工作流服务，忽略工作流服务刷新"
fi

echo "企业服务初始化结束"
