package main.common.config;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Condition;
import org.springframework.context.annotation.ConditionContext;
import org.springframework.core.type.AnnotatedTypeMetadata;

import com.justep.service.config.api.ServiceConfig;
import com.justep.service.config.api.ServiceConfigFactory;

public class DatabaseCondition implements Condition {
	private static final Logger logger = LoggerFactory.getLogger(DatabaseCondition.class);
	/**
	 * 架构服务名称
	*/
	private String archServiceName = "main";
	/**
	 * 环境变量前缀
	 */
	private String envPrefix = "MAIN";

	@Override
	public boolean matches(ConditionContext arg0, AnnotatedTypeMetadata arg1) {
		
		logger.info("检查是否启用服务数据库配置");
		ServiceConfig config = ServiceConfigFactory.get(archServiceName);
		String dbHost = config.getVar(envPrefix + "_DBHOST");
		String dbType = config.getVar(envPrefix + "_DBTYPE");
		String dbName = config.getVar(envPrefix + "_DBNAME");
		String dbPort = config.getVar(envPrefix + "_DBPORT");
		String dbSchema = config.getVar(envPrefix + "_DBSCHEMA");
		String username = config.getVar(envPrefix + "_DBUSER");
		String password = config.getVar(envPrefix + "_DBPASSWD");
		String properties = config.getVar(envPrefix + "_DBPROPERTIES");

		logDBProp(dbType, dbHost, dbPort, dbName, dbSchema, username, password, properties);

		if (StringUtils.isEmpty(dbHost)) {
			logger.info("服务: {} 未定义数据库参数,禁用数据库配置", archServiceName);
			return false;
		} else {
			logger.info("服务: {} 定义数据库参数,启用数据库配置", archServiceName);
			return true;
		}
		
	}

	private void logDBProp(String dbType, String dbHost, String dbPort, String dbName, String dbSchema, String username,
			String password, String prop) {
		StringBuilder buf = new StringBuilder();
		buf.append("dbType: ").append(dbType).append("\n");
		buf.append("dbHost: ").append(dbHost).append("\n");
		buf.append("dbPort: ").append(dbPort).append("\n");
		buf.append("dbName: ").append(dbName).append("\n");
		buf.append("dbSchema: ").append(dbSchema).append("\n");
		buf.append("username: ").append(username).append("\n");
		buf.append("password: ").append(password).append("\n");
		buf.append("prop: ").append(prop).append("\n");
		logger.info("服务数据库参数, {} \n{}", archServiceName, buf.toString());
	}

}
