package servicemetainfo.common;

import java.sql.Connection;
import java.util.Map;
import java.util.Properties;
import java.math.BigDecimal;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Invocation;
import org.apache.ibatis.plugin.Plugin;
import org.apache.ibatis.plugin.Signature;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.util.JdbcConstants;

import servicemetainfo.common.config.SpringContextUtils;

@Intercepts({@Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class}),
	@Signature(type = Executor.class, method = "query", args = {MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class})})
public class PageInterceptor implements Interceptor {

	@Override
	public Object intercept(Invocation invocation) throws Throwable {
		if (PluginUtil.processTarget(invocation.getTarget()) instanceof StatementHandler) {
			handleStatementHandler(invocation);
		}
		return invocation.proceed();
	}

	private void handleStatementHandler(Invocation invocation) {
		StatementHandler statementHandler = (StatementHandler) PluginUtil.processTarget(invocation.getTarget());
		MetaObject metaObject = SystemMetaObject.forObject(statementHandler);
		MappedStatement mappedStatement = (MappedStatement) metaObject.getValue("delegate.mappedStatement");
		if (!SqlCommandType.SELECT.equals(mappedStatement.getSqlCommandType())) {
            return;
        }
		String mapId = mappedStatement.getId();
		boolean isPageable = mapId.contains(".queryPagedEntity") && mapId.endsWith(MybatisPage.PageQuerySuffix);
		isPageable = isPageable || mapId.endsWith(".selectPage") || mapId.endsWith(".selectMapsPage");
		if (isPageable  && isPage(statementHandler.getParameterHandler().getParameterObject())) {
			Object paraObject = statementHandler.getBoundSql().getParameterObject();
			PageRequest pageRequest = null;
			Page pageInfo = null;
			if (paraObject instanceof Pageable) {
				pageRequest = (PageRequest) paraObject;
			}
			if (paraObject instanceof Map) {
				Map<String, Object> paraMap = (Map<String, Object>) paraObject;
				if (paraMap.containsKey("pageRequest"))
				pageRequest = (PageRequest) paraMap.get("pageRequest");
				if (paraMap.containsKey("page"))
					pageInfo = (Page) paraMap.get("page");
			}
			/**
			 * 不需要分页的场合，如果 size 小于 0 返回结果集
			 */
			if (null == pageRequest || pageRequest.getPageSize() < 0) {
			    if (pageInfo == null || pageInfo.getSize() < 0) {
	            return;
	        }
	        }
			long pageOffset = 0;
			long pageSize = 0;
			if (pageInfo != null) {
				pageSize = pageInfo.getSize();
				pageOffset = (pageInfo.getCurrent() - 1) * pageSize;
			} else {
				pageOffset = pageRequest.getOffset();
				pageSize = pageRequest.getPageSize();
			}
			String sql = (String) metaObject.getValue("delegate.boundSql.sql");
			sql = sql.trim();
			DruidDataSource ds = (DruidDataSource) SpringContextUtils.getBeanById("servicemetainfo-moduleDataSource");
			String dbtype = ds.getDbType();
			Connection conn = null;
			try {
				conn = ds.getConnection();
				String dbDriverName = conn.getMetaData().getDriverName();
			    System.out.println("===DbDriverName: " + dbDriverName);
				if (dbDriverName.indexOf("PostgreSQL") >= 0) dbtype = JdbcConstants.POSTGRESQL.name();
				else if (dbDriverName.indexOf("MySQL") >= 0) dbtype = JdbcConstants.MYSQL.name();
				else if (dbDriverName.indexOf("SQL Server") >= 0) dbtype = JdbcConstants.SQL_SERVER.name();
				else if (dbDriverName.indexOf("Oracle") >= 0) dbtype = JdbcConstants.ORACLE.name();
			} catch (Exception e) { 
                System.out.println(e.getMessage());
            } finally {
                if (conn != null) {
                    try { conn.close(); } catch(Exception e) {}
                }
            }

			String pageSql = sql;
			if (StringUtils.equals(dbtype, JdbcConstants.POSTGRESQL.name()) || StringUtils.equals(dbtype, JdbcConstants.MYSQL.name()) || StringUtils.equals(dbtype, JdbcConstants.MARIADB)) {
				pageSql = sql + " LIMIT " + pageSize + " OFFSET " + pageOffset;
			} else if (StringUtils.equals(dbtype, JdbcConstants.SQL_SERVER.name()) || StringUtils.equals(dbtype, JdbcConstants.JTDS.name())){
				pageSql = getSqlServersql(sql, pageOffset, pageSize);
			} else if (StringUtils.equals(dbtype, JdbcConstants.ORACLE.name())) {
				pageSql = "SELECT * FROM ( SELECT TMP.*, ROWNUM ROW_ID FROM ( " +
						sql + " ) TMP WHERE ROWNUM <=" + (pageOffset+pageSize) + ") WHERE ROW_ID > " + pageOffset;
			}
			System.out.println("===DbType: " + dbtype + ", PageSql:" + pageSql);
			metaObject.setValue("delegate.boundSql.sql", pageSql);
			metaObject.setValue("delegate.rowBounds.offset", RowBounds.NO_ROW_OFFSET);
			metaObject.setValue("delegate.rowBounds.limit", RowBounds.NO_ROW_LIMIT);
		}
	}

	@Override
	public Object plugin(Object target) {
		if (Executor.class.isAssignableFrom(target.getClass())) {
			PageExecutor executor = new PageExecutor((Executor)target);
			return Plugin.wrap(executor, this);
		}
		if (target instanceof StatementHandler) {
			return Plugin.wrap(target, this);
		}
		return target;
	}

	@Override
	public void setProperties(Properties properties) {
		
	}
	
	private String getSqlServersql(String originalSql, long pageOffset, long pageSize) {
       String orderby = getOrderByPart(originalSql);
       // if no ORDER BY is specified use fake ORDER BY field to avoid errors
       String sqlPartString = originalSql;
       if (StringUtils.isEmpty(orderby)) {
           orderby = "ORDER BY CURRENT_TIMESTAMP";
       } else {
           sqlPartString = originalSql.substring(0, originalSql.toLowerCase().indexOf("order by"));
       }
       long firstParam = pageOffset + 1;
       long secondParam = pageOffset + pageSize;
       String pageSql = "WITH selectTemp AS (SELECT TOP 100 PERCENT " +
            " ROW_NUMBER() OVER (" + orderby + ") as __row_number__, * FROM (" + sqlPartString +
            ")___TMP_TABLE___) SELECT * FROM selectTemp WHERE __row_number__ BETWEEN " +
            //原因：mysql中limit 10(offset,size) 是从第10开始（不包含10）,；而这里用的BETWEEN是两边都包含，所以改为offset+1
            firstParam + " AND " + secondParam + " ORDER BY __row_number__";
        return pageSql;
	}
	
    private static String getOrderByPart(String sqlIn) {
        String sql = sqlIn.replaceAll("\n", " ");
        System.out.println("===getOrderByPart:" + sql);
        String loweredString = sql.toLowerCase();
        int orderByIndex = loweredString.indexOf("order by");
        if (orderByIndex != -1) {
            String orderBy = sql.substring(orderByIndex);
            int fromIndex = loweredString.indexOf("from ");
            String selectFields = sql.substring(7, fromIndex -1);
            String loweredSelectFields = selectFields.toLowerCase();
            String[] arr = orderBy.split(",");
            StringBuilder sb = new StringBuilder(orderBy.length());
            for (int i=0; i<arr.length; i++) {
                String item = arr[i];
                String[] arr2 = item.split("[\\s\\t\\n\\r]+");
                for (int j=0; j<arr2.length; j++) {
                    String item2 = arr2[j];
                    if (item2.equalsIgnoreCase("order") || item2.equalsIgnoreCase("by") || item2.equalsIgnoreCase("asc") || item2.equalsIgnoreCase("desc")) {
                        sb.append(arr2[j]).append(" ");
                        continue;
                    }
                    int start = -1;
                    if (item2.indexOf(".") > 0) {
                        String[] arr3 = item2.split("\\.");
                        start = loweredSelectFields.indexOf(arr3[1] + " ");
                    } else {
                        start = loweredSelectFields.indexOf(item2 + " ");
                    }

                        if (start >= 0) {
                            int end = loweredSelectFields.substring(start).indexOf(",");
                            String[] field;
                            if (end > 0) { // not the last field
                                field = selectFields.substring(start, start + end).split("[\\s\\t\\n\\r]+");
                            } else {
                                field = selectFields.substring(start).split("[\\s\\t\\n\\r]+");
                            }
                            arr2[j] = field[field.length-1];
                        }
                    sb.append(arr2[j]).append(" ");
                }
                if (i != arr.length -1) {
                    sb.append(",");
                }
            }
            return sb.toString();
        } else {
            return StringUtils.EMPTY;
        }
    }
 	/**
     * 是否分页
     *
     * @param parameterObject 参数对象
     * @return 分页参数
     */
    public static boolean isPage(Object parameterObject) {
        if (parameterObject != null) {
            if (parameterObject instanceof Map) {
                Map<?, ?> parameterMap = (Map<?, ?>) parameterObject;
                for (Map.Entry entry : parameterMap.entrySet()) {
                    if (entry.getValue() != null && entry.getValue() instanceof com.justep.cloud.repo.PageRequest) {
                        return true;
                    }
                }
            } else if (parameterObject instanceof com.justep.cloud.repo.PageRequest) {
                return true;
            }
        }
        return false;
    }
}
