package servicemetainfo.common.config;

import java.util.List;
import java.util.ArrayList;
import java.util.Properties;
import java.util.Map;
import java.util.HashMap;
import java.util.TimeZone;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.lang.StringUtils;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Conditional;
import org.springframework.context.annotation.Configuration;

import com.alibaba.druid.filter.Filter;
import com.alibaba.druid.filter.logging.Log4jFilter;
import com.alibaba.druid.filter.stat.StatFilter;
import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidDataSourceFactory;
import com.alibaba.druid.pool.vendor.MySqlValidConnectionChecker;
import com.alibaba.druid.util.JdbcUtils;
import com.google.common.collect.Lists;
import com.justep.util.db.DatabaseType;
import com.justep.util.db.JDBCUrl;
import com.justep.service.config.api.ServiceConfig;
import com.justep.service.config.api.ServiceConfigFactory;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.beans.factory.annotation.Autowired;
import com.justep.comp.util.DataSourceUtil;


import org.apache.ibatis.plugin.Interceptor;
import com.baomidou.mybatisplus.extension.spring.MybatisSqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.springframework.core.io.ClassPathResource;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.mybatis.spring.mapper.MapperScannerConfigurer;
import org.mybatis.spring.SqlSessionTemplate;
import org.apache.ibatis.session.SqlSessionFactory;



@Configuration(value="servicemetainfo-databaseConfig")
@Conditional(value = { DatabaseCondition.class })
public class DatabaseConfig  extends MybatisPlugInConfig{
	private static final Logger logger = LoggerFactory.getLogger(DatabaseConfig.class);


	class DBResource {
		private String url;
		private String dialect;
		private String username;
		private String password;
		private String driverClassName;
		private String dbType;
		private String dbName;
		private String dbNameIsSID;
		private String properties;
		private String dbHost;
		private String dbPort;
		private String dbSchema;
		private String validateSql;
        private String shardingConfig;
		private Properties connProp;

		private Element root;
		private boolean fromEnv = false;
		private boolean throughDbProxy = true;

		public DBResource() throws Exception {
			ServiceConfig config = ServiceConfigFactory.get(archServiceName);
			dbHost = config.getVar(envPrefix + "_DBHOST");
			if (StringUtils.isBlank(dbHost)) {
				throw new IllegalStateException("服务未定义数据库参数: " + archServiceName);
			} else {
				dbType = config.getVar(envPrefix + "_DBTYPE");
				dbName = config.getVar(envPrefix + "_DBNAME");
				dbPort = config.getVar(envPrefix + "_DBPORT");
				dbSchema = config.getVar(envPrefix + "_DBSCHEMA");
				username = config.getVar(envPrefix + "_DBUSER");
				password = config.getVar(envPrefix + "_DBPASSWD");
				properties = config.getProp(envPrefix + "_DBPROPERTIES");
                shardingConfig = config.getVar(envPrefix + "_DBEXTSHARDING");
				dbNameIsSID = config.getVar(envPrefix + "_DBNAMEISSID");
				fromEnv = true;
                String envUseDbProxy = config.getVar("COMP_DBPROXY_USEDBPROXY");
		        if (envUseDbProxy != null && (envUseDbProxy.equalsIgnoreCase("true") || envUseDbProxy.equalsIgnoreCase("yes") || envUseDbProxy.equalsIgnoreCase("1"))) {
			        throughDbProxy = true;
		        } else {
		            throughDbProxy = false;
                }

            }
			init();
            logger.info("db resource url from env {}, url: {}, validateSql: {}, driver: {}", fromEnv, url, validateSql, driverClassName);
		}

		private void init() throws Exception {
			if (fromEnv) {
				connProp = genConnProp();
				url = genJDBCUrl();
				DatabaseType dt = DatabaseType.getDatabaseType(url);
				validateSql = DatabaseType.getValidationSql(dt);
				driverClassName = dt.getDriverClass();
				dialect = dt.getDialect();
				if (!StringUtils.isEmpty(shardingConfig)) {
				    url += DatabaseType.getItemSeparator(dt) + "shardingConfig=" + shardingConfig;
				}
                if (useDbProxy()) {
				    driverClassName = "com.justep.dbproxy.ProxyDriver";
                }
			} else {
				throw new IllegalStateException("服务未定义数据库参数: " + archServiceName);
			}
		}
        public boolean useDbProxy() {
            if (url.indexOf("dbproxy=false") < 0 && throughDbProxy) {
                return true;
            }
            return false;
        }
		private Properties genConnProp() {
			Properties ret = new Properties();
			if (dbType.contains("oracle")) {
				ret.setProperty("oracle.jdbc.timezoneAsRegion", "false");
			}
			return ret;
		}

		private String genJDBCUrl() {
			JDBCUrl jdbcUrl = new JDBCUrl(DatabaseType.parse(dbType), dbHost, dbPort, dbName, username, password, dbSchema, properties);
        	jdbcUrl.setWithUserPassword(false);
			if (StringUtils.equalsIgnoreCase("false", dbNameIsSID)) {
				jdbcUrl.setDbNameIsSID(false);
			}
        	return jdbcUrl.jdbcUrl();
		}

		public String getUrl() {
			return url;
		}

		public void setUrl(String url) {
			this.url = url;
		}

		public String getDialect() {
			return dialect;
		}

		public void setDialect(String dialect) {
			this.dialect = dialect;
		}

		public String getUsername() {
			return username;
		}

		public void setUsername(String username) {
			this.username = username;
		}

		public String getPassword() {
			return password;
		}

		public void setPassword(String password) {
			this.password = password;
		}

		public String getDriverClassName() {
			return driverClassName;
		}

		public void setDriverClassName(String driverClassName) {
			this.driverClassName = driverClassName;
		}
		public String getValidateSql() {
			return validateSql;
		}

		public Properties getConnProp() {
			return connProp;
		}
	}

	private String serviceName = "servicemetainfo";
	private String envPrefix = "SERVICE_META_INFO";
	private String archServiceName = "service-meta-info";
	
	private DBResource resource;

	public DatabaseConfig() throws Exception {
		resource = new DBResource();
	}

	@Bean(value = "servicemetainfo-druidLogFilter")
	public Log4jFilter druidLogFilter() {
		Log4jFilter filter = new Log4jFilter();
		filter.setStatementLogErrorEnabled(false);
		filter.setStatementExecutableSqlLogEnable(false);
		return filter;
	}

    private static String CONN_MAX_ACTIVE="DRUID_CONNECTION_MAX_ACTIVE";
    private static String REMOVE_ABANDONED_TIMEOUT="DRUID_REMOVE_ABANDONED_TIMEOUT";
	private static String getMaxActive() {
		String ret = System.getenv(CONN_MAX_ACTIVE);
		if(ret == null || ret.length() == 0) {
			ret = System.getProperty(CONN_MAX_ACTIVE);
		}
		if(ret == null || ret.length() == 0) {
            ret = "150";
		}
		return ret;
	}
    private static String getRemoveAbandonedTimeout() {
		String ret = System.getenv(REMOVE_ABANDONED_TIMEOUT);
		if(ret == null || ret.length() == 0) {
			ret = System.getProperty(REMOVE_ABANDONED_TIMEOUT);
		}
		if(ret == null || ret.length() == 0) {
            ret = "300";
		}
		return ret;
    }

	boolean datasourceCreated = false;
	String datasourceModule = null;

	@Bean(value = "servicemetainfo-moduleDataSource", initMethod = "init", destroyMethod = "close")
	public DruidDataSource moduleDataSource(ApplicationContext ctx) {
		Map<String, DruidDataSource> dsmap = ctx.getBeansOfType(DruidDataSource.class);
		for (String name : dsmap.keySet()) {
			if(!isAutoCreate(name)) {
				continue;
			}
			DruidDataSource ds = dsmap.get(name);
			if (same(ds, this.resource)) {
				String prevdsName = ds.getName();
				ds.setName(DataSourceUtil.usedDatasourceName(prevdsName, archServiceName));
				String[] parts = StringUtils.split(name, "-");
				datasourceModule = parts[0];
				datasourceCreated = false;
				logger.info("{} 类中, 服务: {} 使用已经存在的datasource: {} , 创建该datasource的模块: {}", this.getClass().getName(),
						this.archServiceName, name, this.datasourceModule);
				setDatasourceCreateStatus(datasourceModule,datasourceCreated);		
				return ds;
			}
		}
		logger.info("{} 类中, 服务: {} 创建新的datasource", this.getClass().getName(), archServiceName);
		datasourceCreated = true;
		datasourceModule = null;
		setDatasourceCreateStatus(datasourceModule,datasourceCreated);
		DruidDataSource ret = doCreateDatasource();
		ret.setName(DataSourceUtil.generatedDatasourceName(archServiceName));
		return ret;
	}
	private boolean isAutoCreate(String dsname) {
		int idx = StringUtils.indexOf(dsname, "-");
		if (idx == -1) {
			return false;
		}
		return StringUtils.endsWith(dsname, "moduleDataSource");

	}
	private void setDatasourceCreateStatus(String datasourceModule, boolean datasourceCreated) {
		String createdKey = archServiceName+"_datasourceCreated";
		String moduleKey = archServiceName+"_datasourceModule";
		System.setProperty(createdKey, Boolean.toString(datasourceCreated));
		String moduleVal = datasourceModule;
		if(StringUtils.isEmpty(moduleVal)) {
			moduleVal = "";
		}
		System.setProperty(moduleKey, moduleVal);
	}
	private boolean same(DruidDataSource ds, DBResource res) {
		boolean urlcond = StringUtils.equals(ds.getUrl(), res.getUrl());
		boolean usercond = StringUtils.equals(ds.getUsername(), res.getUsername());
		return urlcond && usercond;
	}
	private DruidDataSource doCreateDatasource() {
		try{
            String maxActive = getMaxActive();
			Map<String, String> map = new HashMap<String, String>();
			map.put("init", "false");
			map.put("url", resource.getUrl());
			map.put("minIdle", "1");
			map.put("maxActive", maxActive);
			map.put("testOnBorrow", "false");
			map.put("testOnReturn", "false");
			map.put("testWhileIdle", "true");
			DruidDataSource datasource = (DruidDataSource) DruidDataSourceFactory.createDataSource(map);
	
			datasource.setUrl(resource.getUrl());
			datasource.setUsername(resource.getUsername());
			datasource.setPassword(resource.getPassword());
			datasource.setDriverClassName(resource.getDriverClassName());
            if (resource.useDbProxy()) {
                datasource.setValidConnectionChecker(new MySqlValidConnectionChecker());
	        }
			// configuration
			datasource.setInitialSize(1);
			datasource.setMinIdle(1);
			datasource.setMaxActive(Integer.parseInt(maxActive));
			datasource.setMaxWait(5000);
			datasource.setTimeBetweenEvictionRunsMillis(60000);
			datasource.setMinEvictableIdleTimeMillis(220000);
			datasource.setMaxEvictableIdleTimeMillis(300000);
			datasource.setTestWhileIdle(true);
			datasource.setTestOnBorrow(false);
			datasource.setTestOnReturn(false);

			datasource.setPoolPreparedStatements(false);
			datasource.setValidationQuery(resource.getValidateSql());
			
			datasource.setConnectionErrorRetryAttempts(1);
			datasource.setInitExceptionThrow(false);
			datasource.setTimeBetweenConnectErrorMillis(6000);

            datasource.setRemoveAbandoned(true);
            String removeAbandonedTimeout = getRemoveAbandonedTimeout();
            datasource.setRemoveAbandonedTimeout(Integer.parseInt(removeAbandonedTimeout));
            datasource.setLogAbandoned(true);

			List<Filter> filters = Lists.newArrayList(druidLogFilter());
			StatFilter statFilter = new StatFilter();
			statFilter.setLogSlowSql(true);
			filters.add(statFilter);
			datasource.setProxyFilters(filters);

			datasource.setConnectProperties(resource.getConnProp());

			
			if (datasource.getConnectProperties().getProperty("socketTimeout") == null || datasource.getUrl().indexOf("socketTimeout") < 0) {
				datasource.setSocketTimeout(600000);
			}
            datasource.init();
			
			return datasource;
		}catch(Exception ex){
			logger.error("datasource error",ex);
			throw new RuntimeException(ex);
		}
	}

	
	
	@Bean(value="servicemetainfo-sqlSessionFactory")
   public MybatisSqlSessionFactoryBean sqlSessionFactory(@Qualifier("servicemetainfo-moduleDataSource") DruidDataSource dataSource) throws Exception {
        MybatisSqlSessionFactoryBean mybatisPlus = new MybatisSqlSessionFactoryBean();
		mybatisPlus.setConfigLocation(new ClassPathResource("service-meta-info-spring-mybatis.xml"));
		mybatisPlus.setPlugins(new Interceptor[] {mybatisPlusInterceptor()});
        mybatisPlus.setDataSource(dataSource);  
        return mybatisPlus;
    }
    
    
	@Bean(value="servicemetainfo-mapperScannerConfigurer2")
	public MapperScannerConfigurer mapperScannerConfigurer2(@Qualifier("servicemetainfo-sqlSessionFactory") SqlSessionFactory sqlSessionFactory) {
		DatabaseType dbtype = DatabaseType.parse(resource.dbType);
		sqlSessionFactory.getConfiguration().setDatabaseId(dbtype.name());
		MapperScannerConfigurer ret = new MapperScannerConfigurer();
		ret.setBasePackage("servicemetainfo.**.dao.**");
		ret.setSqlSessionFactoryBeanName("servicemetainfo-sqlSessionFactory");
		ret.setNameGenerator(new MapperNameGenerator());
		return ret;
	}
	
	
	@Bean(value="servicemetainfo-moduleTransactionManager")
	public PlatformTransactionManager moduleTransactionManager(
			@Qualifier("servicemetainfo-moduleDataSource") DruidDataSource datasource, ApplicationContext ctx) {
		logger.info("{} 类中 , datasourceCreated: {} , datasourceModule: {}", this.getClass().getName(),
				datasourceCreated, datasourceModule);
		if (datasourceCreated) {
			logger.info("{} 类中创建了Datasource,创建新的 DataSourceTransactionManager", this.getClass().getName());
			DataSourceTransactionManager ret = new DataSourceTransactionManager();
			ret.setDataSource(datasource);
			return ret;
		} else {
			String checkkey = datasourceModule + "-moduleTransactionManager";
			logger.info("{} 类中使用已有的 DataSourceTransactionManager: {}", this.getClass().getName(), checkkey);
			try {
				PlatformTransactionManager ret = ctx.getBean(checkkey, PlatformTransactionManager.class);
				return ret;
			} catch (Exception ex) {
				String msg = String.format("%s 类中 , 服务: %s 使用已经存在的datasource,但无对应的 PlatformTransactionManager: %s\n",
						this.getClass().getName(), this.archServiceName, checkkey);
				throw new RuntimeException(msg, ex);
			}

		}

	}
	@Bean(name = "servicemetainfo-sqlSessionTemplate")
    public SqlSessionTemplate testSqlSessionTemplate(@Qualifier("servicemetainfo-sqlSessionFactory") SqlSessionFactory sqlSessionFactory) throws Exception {
        return new SqlSessionTemplate(sqlSessionFactory);
    }
	
	
}
