package servicemetainfo.common.config;

import javax.annotation.PreDestroy;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.SmartInitializingSingleton;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.context.annotation.Conditional;

import com.justep.util.event.DbAbstractEvent;

@Conditional(value = { EventListenerProcessorCondition.class })
@Configuration(value="servicemetainfo-eventListenerProcessor")
@Order(Ordered.HIGHEST_PRECEDENCE)
public class EventListenerProcessor implements ApplicationContextAware, SmartInitializingSingleton {
	private AbstractApplicationContext applicationContext;
	private AbstractApplicationContext parentApplicationContext;
    private ApplicationListener<DbAbstractEvent> parentApplicationListener;

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = (AbstractApplicationContext) applicationContext;
		this.parentApplicationContext = (AbstractApplicationContext) applicationContext.getParent();
	}

	@Override
	public void afterSingletonsInstantiated() {
		final AbstractApplicationContext currentApplicationContext = this.applicationContext;
		this.applicationContext.addApplicationListener(new ApplicationListener<DbAbstractEvent>() {
			@Override
			public void onApplicationEvent(DbAbstractEvent event) {
				event.getJarsLinkDispatchers().add(currentApplicationContext);
			}
		});

		if(this.parentApplicationContext!=null) {
			this.parentApplicationListener = new ApplicationListener<DbAbstractEvent>() {
				@Override
				public void onApplicationEvent(DbAbstractEvent event) {
					// 自己模块未处理过
					if(!event.getJarsLinkDispatchers().contains(currentApplicationContext)) {
						event.getJarsLinkDispatchers().add(currentApplicationContext);
						if(currentApplicationContext.isActive()) {
							currentApplicationContext.publishEvent(event);
						}
					}
				}
			};
			this.parentApplicationContext.addApplicationListener(this.parentApplicationListener);
		}
		
	}

	@PreDestroy
	public void close() {
		if(this.parentApplicationContext!=null) {
			this.parentApplicationContext.getApplicationListeners().remove(this.parentApplicationListener);
		}
	}
}