package servicemetainfo.common.config;

import java.io.File;
import java.nio.file.Paths;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Condition;
import org.springframework.context.annotation.ConditionContext;
import org.springframework.core.type.AnnotatedTypeMetadata;

public class EventListenerProcessorCondition implements Condition {
	private static final Logger logger = LoggerFactory.getLogger(EventListenerProcessorCondition.class);
	/**
	 * 架构服务名称
	 */
	private String archServiceName = "service-meta-info";

	@Override
	public boolean matches(ConditionContext context, AnnotatedTypeMetadata metadata) {
		boolean ret = checkRuntime("tomcat");
		if(ret) {
			logger.info("服务: {} 启用 EventListenerProcessor",archServiceName);
		} else {
			logger.info("服务: {} 禁用 EventListenerProcessor",archServiceName);
		}
		return ret;
	}

	private boolean checkRuntime(String runtime) {
		String home = getPropertyOrEnv("JUSTEP_HOME");
		if (StringUtils.isEmpty(home)) {
			throw new RuntimeException("系统属性和环境变量未定义: JUSTEP_HOME");
		}
		String name = archServiceName + ".config.json";
		File fconf = null;
		if (isWindows()) {
			fconf = Paths.get(home, "dist", runtime, "conf", name).toFile();
		} else {
			fconf = Paths.get("/dist", runtime, "conf", name).toFile();
		}
		return fconf.exists();
	}

	static String getPropertyOrEnv(String name) {
		String ret = System.getProperty(name);
		if (StringUtils.isEmpty(ret)) {
			ret = System.getenv(name);
		}
		return com.justep.util.SecurityUtil.encodeAndCheck(ret);
	}

	static public boolean isWindows() {
		String os = System.getProperty("os.name");
		return os.contains("Windows");
	}
}
