package servicemetainfo.common.config;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.BeanFactoryUtils;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.context.ApplicationContext;
import org.springframework.core.annotation.AnnotationAwareOrderComparator;
import org.springframework.jdbc.UncategorizedSQLException;
import org.springframework.orm.ObjectOptimisticLockingFailureException;
import org.springframework.web.multipart.MultipartResolver;
import org.springframework.web.servlet.DispatcherServlet;
import org.springframework.web.servlet.FlashMapManager;
import org.springframework.web.servlet.HandlerAdapter;
import org.springframework.web.servlet.HandlerExceptionResolver;
import org.springframework.web.servlet.HandlerExecutionChain;
import org.springframework.web.servlet.HandlerMapping;
import org.springframework.web.servlet.LocaleResolver;
import org.springframework.web.servlet.RequestToViewNameTranslator;
import org.springframework.web.servlet.ThemeResolver;
import org.springframework.web.servlet.ViewResolver;

import com.alibaba.fastjson.JSONObject;
import com.justep.lang.exception.BaseRuntimeException;

import servicemetainfo.common.OptimisticLockingFailureException;

public class MyDispatcher {
	
	protected static final Logger logger = LoggerFactory.getLogger(MyDispatcher.class);
	
	/** Detect all HandlerMappings or just expect "handlerMapping" bean? */
	private boolean detectAllHandlerMappings = true;

	/** Detect all HandlerAdapters or just expect "handlerAdapter" bean? */
	private boolean detectAllHandlerAdapters = true;

	/** Detect all HandlerExceptionResolvers or just expect "handlerExceptionResolver" bean? */
	private boolean detectAllHandlerExceptionResolvers = true;

	/** Detect all ViewResolvers or just expect "viewResolver" bean? */
	private boolean detectAllViewResolvers = true;

	/** Throw a NoHandlerFoundException if no Handler was found to process this request? **/
	private boolean throwExceptionIfNoHandlerFound = false;

	/** Perform cleanup of request attributes after include request? */
	private boolean cleanupAfterInclude = true;

	/** MultipartResolver used by this servlet */
	private MultipartResolver multipartResolver;

	/** LocaleResolver used by this servlet */
	private LocaleResolver localeResolver;

	/** ThemeResolver used by this servlet */
	private ThemeResolver themeResolver;

	/** List of HandlerMappings used by this servlet */
	private List<HandlerMapping> handlerMappings;

	/** List of HandlerAdapters used by this servlet */
	private List<HandlerAdapter> handlerAdapters;

	/** List of HandlerExceptionResolvers used by this servlet */
	private List<HandlerExceptionResolver> handlerExceptionResolvers;

	/** RequestToViewNameTranslator used by this servlet */
	private RequestToViewNameTranslator viewNameTranslator;

	/** FlashMapManager used by this servlet */
	private FlashMapManager flashMapManager;

	/** List of ViewResolvers used by this servlet */
	private List<ViewResolver> viewResolvers;
	
	public MyDispatcher(ApplicationContext context) {
		initMultipartResolver(context);
		initLocaleResolver(context);
		initHandlerMappings(context);
		initHandlerAdapters(context);
//		initRequestToViewNameTranslator(context);
		initHandlerExceptionResolvers(context);
//		initViewResolvers(context);
	}
	
	private void initMultipartResolver(ApplicationContext context) {
		try {
			this.multipartResolver = context.getBean(DispatcherServlet.MULTIPART_RESOLVER_BEAN_NAME, MultipartResolver.class);
			if (logger.isDebugEnabled()) {
				logger.debug("Using MultipartResolver [" + this.multipartResolver + "]");
			}
		}
		catch (NoSuchBeanDefinitionException ex) {
			// Default is no multipart resolver.
			this.multipartResolver = null;
			if (logger.isDebugEnabled()) {
				logger.debug("Unable to locate MultipartResolver with name '" +DispatcherServlet. MULTIPART_RESOLVER_BEAN_NAME +
						"': no multipart request handling provided");
			}
		}
	}
	
	private void initLocaleResolver(ApplicationContext context) {
		try {
			this.localeResolver = context.getBean(DispatcherServlet.LOCALE_RESOLVER_BEAN_NAME, LocaleResolver.class);
			if (logger.isDebugEnabled()) {
				logger.debug("Using LocaleResolver [" + this.localeResolver + "]");
			}
		}
		catch (NoSuchBeanDefinitionException ex) {
			// We need to use the default.
			this.localeResolver = null;
			if (logger.isDebugEnabled()) {
				logger.debug("Unable to locate LocaleResolver with name '" + DispatcherServlet.LOCALE_RESOLVER_BEAN_NAME +
						"': using null");
			}
		}
	}
	
	private void initHandlerMappings(ApplicationContext context) {
		this.handlerMappings = null;

		if (this.detectAllHandlerMappings) {
			// Find all HandlerMappings in the ApplicationContext, including ancestor contexts.
			Map<String, HandlerMapping> matchingBeans =
					BeanFactoryUtils.beansOfTypeIncludingAncestors(context, HandlerMapping.class, true, false);
			if (!matchingBeans.isEmpty()) {
				this.handlerMappings = new ArrayList<HandlerMapping>(matchingBeans.values());
				// We keep HandlerMappings in sorted order.
				AnnotationAwareOrderComparator.sort(this.handlerMappings);
			}
		}
		else {
			try {
				HandlerMapping hm = context.getBean(DispatcherServlet.HANDLER_MAPPING_BEAN_NAME, HandlerMapping.class);
				this.handlerMappings = Collections.singletonList(hm);
			}
			catch (NoSuchBeanDefinitionException ex) {
				// Ignore, we'll add a default HandlerMapping later.
			}
		}

		// Ensure we have at least one HandlerMapping, by registering
		// a default HandlerMapping if no other mappings are found.
		if (this.handlerMappings == null) {
			this.handlerMappings = null;
			if (logger.isDebugEnabled()) {
				logger.debug("No HandlerMappings found in 'mydispatcher': using null");
			}
		}
	}
	
	private void initHandlerAdapters(ApplicationContext context) {
		this.handlerAdapters = null;

		if (this.detectAllHandlerAdapters) {
			// Find all HandlerAdapters in the ApplicationContext, including ancestor contexts.
			Map<String, HandlerAdapter> matchingBeans =
					BeanFactoryUtils.beansOfTypeIncludingAncestors(context, HandlerAdapter.class, true, false);
			if (!matchingBeans.isEmpty()) {
				this.handlerAdapters = new ArrayList<HandlerAdapter>(matchingBeans.values());
				// We keep HandlerAdapters in sorted order.
				AnnotationAwareOrderComparator.sort(this.handlerAdapters);
			}
		}
		else {
			try {
				HandlerAdapter ha = context.getBean(DispatcherServlet.HANDLER_ADAPTER_BEAN_NAME, HandlerAdapter.class);
				this.handlerAdapters = Collections.singletonList(ha);
			}
			catch (NoSuchBeanDefinitionException ex) {
				// Ignore, we'll add a default HandlerAdapter later.
			}
		}

		// Ensure we have at least some HandlerAdapters, by registering
		// default HandlerAdapters if no other adapters are found.
		if (this.handlerAdapters == null) {
			this.handlerAdapters = null;
			if (logger.isDebugEnabled()) {
				logger.debug("No HandlerAdapters found in 'mydispatcher': using null");
			}
		}
	}
	
	private void initHandlerExceptionResolvers(ApplicationContext context) {
		this.handlerExceptionResolvers = null;
		try {
			HandlerExceptionResolver her =
					context.getBean(DispatcherServlet.HANDLER_EXCEPTION_RESOLVER_BEAN_NAME, HandlerExceptionResolver.class);
			this.handlerExceptionResolvers = Collections.singletonList(her);
		}
		catch (NoSuchBeanDefinitionException ex) {
			logger.debug("No HandlerExceptionResolver found in 'mydispatcher': using null");
		}
	}
	
	public void doDispatch(HttpServletRequest request, HttpServletResponse response) throws Exception {
		HandlerExecutionChain mappedHandler = getHandler(request);
		if (mappedHandler == null || mappedHandler.getHandler() == null) {
			processException(request, response, new ServletException("No Mapped Handler found in 'mydispatcher'."),HttpServletResponse.SC_NOT_FOUND);
			return;
		}
		HandlerAdapter ha = getHandlerAdapter(mappedHandler.getHandler());
		ha.handle(request, response, mappedHandler.getHandler());
	}
	
	protected HandlerExecutionChain getHandler(HttpServletRequest request) throws Exception {
		for (HandlerMapping hm : this.handlerMappings) {
			HandlerExecutionChain handler = hm.getHandler(request);
			if (handler != null) {
				return handler;
			}
		}
		return null;
	}
	
	protected HandlerAdapter getHandlerAdapter(Object handler) throws ServletException {
		for (HandlerAdapter ha : this.handlerAdapters) {
			if (ha.supports(handler)) {
				return ha;
			}
		}
		throw new ServletException("No adapter for handler [" + handler +
				"]: Mydispatcher configuration needs to include a HandlerAdapter that supports this handler");
	}

	public void processException(HttpServletRequest request, HttpServletResponse response, Exception ex, Integer status) {
		if (status!=null && status==HttpServletResponse.SC_NOT_FOUND) {
			logger.error(ex.getMessage()+"where url was '"+request.getRequestURI()+"'");
		}else {
			logger.error(ex.getMessage(), ex);
		}
		try {
			response.setCharacterEncoding("UTF-8");
			response.setContentType("application/json");
			Map<String, Object> map = new HashMap<String, Object>();
			if (status == null) {
				if (ex instanceof ObjectOptimisticLockingFailureException) {
					response.setStatus(HttpServletResponse.SC_PRECONDITION_FAILED);
					map.put("status",HttpServletResponse.SC_PRECONDITION_FAILED);
				} else if (ex instanceof UncategorizedSQLException && ex.getCause() instanceof OptimisticLockingFailureException) {
					response.setStatus(HttpServletResponse.SC_PRECONDITION_FAILED);
					map.put("status",HttpServletResponse.SC_PRECONDITION_FAILED);
				} else if (ex instanceof BaseRuntimeException) {
					BaseRuntimeException exception=(BaseRuntimeException)ex;
					response.setStatus(exception.getStatus());
					map.put("status",exception.getStatus());
					map.put("code",exception.getCode());
					map.put("detail", exception.getDetail());
				} else {
					response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
					map.put("status",HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
				}
			}else {
				response.setStatus(status);
				map.put("status", status);
			}
			map.put("message", ex.getMessage());
            // 反射型跨站攻击
			// map.put("url", request.getRequestURI());
			response.getWriter().print(JSONObject.toJSONString(map));
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
