package servicemetainfo.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Locale;
import java.util.concurrent.ExecutionException;

import org.apache.commons.codec.digest.Md5Crypt;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.concurrent.ConcurrentUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.google.common.cache.CacheBuilder;
import com.justep.util.ContextUtil;
import com.justep.util.EncryptUtil;

import servicemetainfo.i18n.I18nResourceLoader;

@RestController
public class MyServiceMetaInfoController {
	
	@Autowired
	ResourceLoader resourceLoader;
	
	
	@RequestMapping(value = "/servicemetainfo/prod", method = RequestMethod.GET)
	public ResponseEntity<String> prod() throws Exception {
		
		
		String justepHome = ContextUtil.getEnv("JUSTEP_HOME");
		File file = new File(justepHome,"model/service/service-meta-info/serviceMetaInfo.json");
		String content = FileUtils.readFileToString(file,"utf-8");
		
		
		HttpHeaders headers = new HttpHeaders();
		headers.set("Content-Type","application/json;charset=UTF-8");
		
		ResponseEntity<String> ret = new ResponseEntity<String>(content,headers,HttpStatus.OK);
		return ret;
	}
	
	
	@GetMapping("/servicemetainfo/i18n/{languageId}/resource.json")
    public ResponseEntity<JSONObject> readFile(@PathVariable String languageId) throws IOException {
		InputStream inputStream =null; 
		try {
	        
	        if(ContextUtil.getEnv("DEPLOY_TYPE").equals("dev")) {
	        	inputStream = new FileInputStream(new File(ContextUtil.getEnv("JUSTEP_HOME") + "/model/service/service-meta-info/src/main/resources/i18n/" + languageId + "/resource.json"));
	            
			}else {
				Resource resource = resourceLoader.getResource("classpath:/i18n/" + languageId + "/resource.json");
		        inputStream = resource.getInputStream();
				
			}
	        
	        JSONObject result = JSON.parseObject(inputStream, JSONObject.class, Feature.DisableSpecialKeyDetect);
	        
	        
	        //  计算资源的MD5值
	        String  expectedMD5  = EncryptUtil.md5(result.toJSONString());

	        //  设置ETag头
	        HttpHeaders  headers  =  new  HttpHeaders();
	        headers.add(HttpHeaders.ETAG,  expectedMD5);
	        
	        return  new  ResponseEntity<>(result,  headers,  HttpStatus.OK);
		}finally {
			IOUtils.close(inputStream);
		}
        
    }
	
	
	@Autowired
	I18nResourceLoader i18nResourceLoader;
	
	@GetMapping("/servicemetainfo/i18n/locale")
    public String readFile(Locale locale,@RequestParam String path) throws IOException, ExecutionException {
		
        return locale.getDisplayLanguage() + locale.getLanguage() + locale.getDisplayCountry();
    }
	
	
	
	
}
