package servicemetainfo.i18n;

import java.util.concurrent.ExecutionException;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.JSONPath;
import com.alibaba.fastjson.parser.Feature;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.justep.util.ContextUtil;
import com.justep.util.SpringWebUtil;

@Component
public class I18nResourceLoader {
	Logger logger = LoggerFactory.getLogger(I18nResourceLoader.class);
	
	@Autowired
	ResourceLoader resourceLoader;

	private final LoadingCache<String, JSONObject> cache;

	public I18nResourceLoader(ResourceLoader resourceLoader) {
		// CacheLoader定义了如何加载缓存
		CacheLoader<String, JSONObject> loader = new CacheLoader<String, JSONObject>() {
			@Override
			public JSONObject load(String languageId) throws Exception {
				
				Resource resource = resourceLoader.getResource("classpath:/i18n/" + languageId + "/resource.json");
				if(resource.exists()) {
					JSONObject result = JSON.parseObject(resource.getInputStream(), JSONObject.class,
							Feature.DisableSpecialKeyDetect);
					return result;
				}else {
					logger.warn("多语言文件" + languageId + "不存在");
					return new JSONObject();
							 
				}
				
			}
		};
		
		if(ContextUtil.getEnv("DEPLOY_TYPE").equals("dev")) {
			this.cache = CacheBuilder.newBuilder().maximumSize(0).softValues().build(loader);
		}else {
			this.cache = CacheBuilder.newBuilder().maximumSize(10).softValues().build(loader);
		}
		
	}

	public String i18n(String key, String languageId) throws ExecutionException {
		
		JSONObject resourceContent = cache.get(languageId);
		String[] keySplits = key.split(">");
		String path = Stream.of(keySplits).map(new Function<String, String>() {
			@Override
			public String apply(String t) {

				return "['" + t.replace("/","\\/") + "']";
			}

		}).collect(Collectors.joining(""));

		if (JSONPath.contains(resourceContent, path)) {
			Object result = JSONPath.eval(resourceContent, path);
			return JSON.toJSONString(result);
		}
		return keySplits[keySplits.length -1];
	}

	public String i18n(String key) throws ExecutionException {

		String languageId = SpringWebUtil.getRequest().getLocale().toLanguageTag();

		return i18n(key,languageId);
	}
	

}
